{$Warnings off}
{$Hints off}
{$AppType Console}

{ By SVSD_VAL }
uses OpenGL15, Objects,wnd_util, textures,
     l_math;

type
  //
  TPlayer  = Record
    Position,
    Size     : TVector;
    Rotate   : Single;
    RotateH  : Single;
  end;

  //
  T2DCamera = Object
    Position : TVector;
    Procedure SetView;
    Procedure SetPosition( P: TVector );
    Function  PointInCamera( P: TVector ): boolean;
    Function  AABBInCamera( Min,Max :TVector ):boolean;
  end;


  Const
  MapSize   = 10;
  MapBlock  : TVector =(X:128;Y:128);

var

  PlayerImg : Array [0..1] of Cardinal;
  groundImg : Array [0..1] of Cardinal;

  Map       : Array [0..MapSize,0..MapSize] of Cardinal;

  Player    : TPlayer;
  Camera    : T2DCamera;

  gfxdir    : string;


Procedure T2DCamera.SetView;
begin
 with Position do
     glTranslateF( -x, -y, 0 );
end;

Procedure T2DCamera.SetPosition( P: TVector );
var S:TVector;
begin
 Position := v_sub(P, vector(width*0.5,height*0.5));
 with Position do
 begin
     S.X := MapBlock.X * (MapSize-1) - Width;
     S.Y := MapBlock.Y * (MapSize-1) - Height;
  if X < 0   then X:= 0;
  if Y < 0   then Y:= 0;
  if X > S.X then X:= S.X;
  if Y > S.Y then Y:= S.Y;
 end;

end;

Function  T2DCamera.PointInCamera( P: TVector ): boolean;
begin
  result :=   (abs(P.x - Position.x -Width  *0.5)< Width  *0.5)
          and (abs(P.y - Position.y -Height *0.5)< Height *0.5);
end;
Function  T2DCamera.AABBInCamera;
begin
Result:=not ( (Position.x > Max.x) or (Position.x+width  < Min.x)
          or  (Position.y > Max.y) or (Position.y+height < Min.y));
end;

Procedure Plane(P,Size:TVector; Rotate:Single);
var
  S :TVector;
begin
  S :=V_Mult(Size,0.5);
 glPushMatrix;
 glTranslateF(P.X,P.Y,0);
 glRotateF( Rotate , 0 ,0,1);

         glBegin(GL_QUADS);
           glTexcoord2f(0, 1); glVertex2f(-S.X , -S.Y);
           glTexcoord2f(1, 1); glVertex2f( S.X , -S.Y);
           glTexcoord2f(1, 0); glVertex2f( S.X ,  S.Y);
           glTexcoord2f(0, 0); glVertex2f(-S.X ,  S.Y);
         glEnd;
           glcolor3f(1,1,1);

 glPopMatrix;
end;

Procedure InitMap;
var X,Y: Integer;
begin
 for Y := 0 to MapSize-1 do
 for X := 0 to MapSize-1 do
 Map[Y,X] := Random(199) div 100;
end;

Procedure RenderMap;
var X,Y, CurTex: Integer;
   tmp : TVector;
begin

 curtex := -1;
 for Y := 0 to MapSize-1 do
 for X := 0 to MapSize-1 do
 begin
  tmp := Vector(  X* MapBlock.X - MapBlock.X*0.5
                 ,Y* MapBlock.y - MapBlock.Y*0.5);


    if Camera.AABBInCamera( tmp , v_add(tmp ,mapblock) ) then
    begin

      if curtex <> Map[y,x] then
      begin
       curtex := Map[y,x];
       glBindTexture(GL_TEXTURE_2D, GroundImg[ curtex ]);
      end;

      Plane( vector(X* MapBlock.x ,Y* MapBlock.y), MapBlock,0);
    end;
 end;

end;

Procedure glInit;
begin
  gfxdir := AddDir(AppDir,'gfx');

  SetProjection(WIDTH,HEIGHT,FALSE); //    3D
  GLClearColor(0.3,0.3,0.3,0);       //     
  glAlphaFunc(GL_GREATER,0.6);
  glDisable(gl_depth_test);


  LoadTGATexture(gfxdir+'player_body.tga', PlayerImg[0]);
  LoadTGATexture(gfxdir+'player_head.tga', PlayerImg[1]);

  LoadTGATexture(gfxdir+'ground.tga' , GroundImg[0]);
  LoadTGATexture(gfxdir+'ground1.tga', GroundImg[1]);

  InitMap;

  Player.Position := Vector(width *0.5, height *0.5);
  Player.Size     := Vector(32,32);


end;

procedure glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  SetProjection(WIDTH,HEIGHT,true);
  glLoadIdentity;

  Camera.SetView;

  RenderMap;


  glEnable(GL_Alpha_Test);
    With Player do
    begin
  glBindTexture(GL_TEXTURE_2D, PlayerImg[0]);
         Plane ( Position, Size,  Rotate);
  glBindTexture(GL_TEXTURE_2D, PlayerImg[1]);
         Plane ( Position, Size,  RotateH);
    end;
  glDisable(GL_Alpha_Test);



end;


Procedure Process;

  Function GetMoveVector(Rotate,Speed:Single):TVector;
  begin
    Result.x :=  sin(Rotate *deg2rad) * Speed * FrameTime *0.01;
    Result.y := -cos(Rotate *deg2rad) * Speed * FrameTime *0.01
  end;
   var
    Mpos: Array [0..1] of Integer;
begin
 glDraw;


 with player do
 begin
   if keys[ ord('W') ] then  Position := v_add( Position , GetMoveVector(Rotate      ,5));
   if keys[ ord('S') ] then  Position := v_add( Position , GetMoveVector(Rotate - 180,5));
   if keys[ ord('A') ] then  RotateH   := RotateH -FrameTime *0.2;
   if keys[ ord('D') ] then  RotateH   := RotateH +FrameTime *0.2;

   if Rotate < RotateH then Rotate := Rotate + FrameTime *0.075;
   if Rotate > RotateH then Rotate := Rotate - FrameTime *0.075;
 end;
 Camera.SetPosition(player.Position);

 GetMPos(@Mpos);
 Write('Mouse position X: ',Mpos[0]:3,' Y: ', Mpos[1]:3,#13);
 Mpos[0]:=100;
 Mpos[1]:=100;
 if keys[vk_shift_l] then SetMPos(@Mpos);
 

 if keys[vk_escape] then done:=true;

end;

begin
 pInit   := @glinit;
 pProcess:= @Process;

 CreateMainLoop;


end.

