unit l_math;
{$Warnings off}
{$Hints On}
interface


const
 rad2deg = 180/pi;
 deg2rad = pi/180;

type
 PVector = ^TVector;
 TVector = record
  X, Y : single;
 end;

 function max(x, y: single): single;
 function min(x, y: single): single;

 function ArcTan2(const Y, X: single): single;
 function ArcCos(const X: single): single;
// 3D
 function Vector(X, Y: single): TVector;
 function V_Add(v1, v2: TVector): TVector;
 function V_Sub(v1, v2: TVector): TVector;
 function V_Mult(v: TVector; d: single): TVector;
 function V_Div(v: TVector; d: single): TVector;
 function V_Dist(v1, v2: TVector): single;
 function V_Distq(v1, v2: TVector): single;
 function V_Length(v: TVector): single;
 function V_Normalize2(v: TVector): TVector;
 function V_Normalize (const v: TVector): TVector;
 function V_Dot(v1, v2: TVector): single;
 function V_Angle(v1, v2: TVector): single;
 function V_Interpolate(v1, v2: TVector; k: single): TVector;
 function V_VC   (v1, v2: TVector): boolean;
 function V_MultV(A,B: TVector): TVector;
 function V_Min(v1,v2: TVector): boolean;


implementation

function max(x, y: single): single;
begin
if x > y then
 Result := x
else
 Result := y;
end;

function min(x, y: single): single;
begin
if x < y then
 Result := x
else
 Result := y;
end;

(***      ***)

// 
function ArcTan2(const Y, X: single): single;
asm
 FLD     Y
 FLD     X
 FPATAN
 FWAIT
end;

function ArcCos(const X: single): single;
begin
Result := ArcTan2(Sqrt(1 - X * X), X);
end;

(***      ***)

//  
function Vector(X, Y: single): TVector;
begin
Result.X := X;
Result.Y := Y;
end;

//  
function V_Add(v1, v2: TVector): TVector;
begin
Result.X := v1.X + v2.X;
Result.Y := v1.Y + v2.Y;
end;


///
function V_VC(v1, v2: TVector): Boolean;
begin
result := (abs(v2.X - v1.X)<0.01) and
          (abs(v2.y - v1.y)<0.01);
end;

//
//  

function V_Sub(v1, v2: TVector): TVector;
begin
Result.X := v1.X - v2.X;
Result.Y := v1.Y - v2.Y;
end;

function V_Mult(v: TVector; d: single): TVector;
begin
Result.X := v.X * d;
Result.Y := v.Y * d;
end;

function V_Div(v: TVector; d: single): TVector;
begin
if d <> 0 then
 Result := Vector(v.X/d, v.Y/d)
else
 Result := v;
end;

function V_MultV(A,B: TVector): TVector;
begin
Result.X := a.X * b.x;
Result.Y := a.Y * b.y;
end;

//    
function V_Dist(v1, v2: TVector): single;
begin
Result := sqrt(sqr(v2.X - v1.X) +
               sqr(v2.Y - v1.Y));
end;

function V_Min(v1,v2: TVector): boolean;
begin
result := (v1.X < v2.X) and
          (v1.Y < v2.Y);
end;

//     
function V_Distq(v1, v2: TVector): single;
begin
Result := sqr(v2.X - v1.X) +
          sqr(v2.Y - v1.Y);
end;

function V_Length(v: TVector): single;
begin
Result := sqrt(sqr(v.X) + sqr(v.Y));
end;

function V_Normalize2(v: TVector): TVector;
begin
Result := V_Mult( v , 1 / V_Length ( v ) );
end;

function V_Normalize(const v: TVector): TVector;
var s : Extended;
begin
s :=  1 / sqrt(sqr(v.X) + sqr(v.Y));
Result.x := v.x * s;
Result.y := v.y * s;
end;

function V_Dot(v1, v2: TVector): single;
begin
Result := v1.X * v2.X + v1.Y * v2.Y;
end;

function V_Angle(v1, v2: TVector): single;
begin
Result := ArcCos(V_Dot(V_Normalize(v1), V_Normalize(v2)));
end;

function V_Interpolate(v1, v2: TVector; k: single): TVector;
begin
Result.X := v1.X + (v2.X - v1.X)/k;
Result.Y := v1.Y + (v2.Y - v1.Y)/k;
end;

end.
