unit Textures;
{$Warnings off}
{By SVSD_VAL}
interface

uses
{$ifdef Win32}
  Windows,
{$endif}
  OpenGL15;

  Type PbyteArray = ^TbyteArray;
       TbyteArray = array [0..1023] of byte;



function LoadTGATexture(Filename: String; var Texture: Cardinal): Boolean;

procedure Texture_Enable(const ID: Cardinal; const Channel: Integer);
procedure Texture_Disable(const Channel: Integer);

implementation
uses wnd_util;

procedure Texture_Enable(const ID: Cardinal; const Channel: Integer);
begin

  glActiveTextureARB(GL_TEXTURE0_ARB + Channel);
  glEnable(GL_TEXTURE_2D);

  if not glIsTexture(ID) then
        glBindTexture(GL_TEXTURE_2D,  0) else
        glBindTexture(GL_TEXTURE_2D, ID);
end;

procedure Texture_Disable(const Channel: Integer);
begin
  glActiveTextureARB(GL_TEXTURE0_ARB + Channel);
  glBindTexture(GL_TEXTURE_2D, 0);
  glDisable(GL_TEXTURE_2D);
end;


{------------------------------------------------------------------}
{  Create the Texture                                              }
{------------------------------------------------------------------}
function CreateTexture(Width, Height, Format : Word; pData : Pointer) : Integer;
var
  Texture : Cardinal;
begin
  glGenTextures(1, @Texture);
  glBindTexture(GL_TEXTURE_2D, Texture);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);      {Texture blends with object background}

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); { only first two can be used }
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); { all of the above can be used }

  case format of
   GL_RGBA,32 : gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, pData);
   GL_RGB ,24 : gluBuild2DMipmaps(GL_TEXTURE_2D, 3, Width, Height, GL_RGB, GL_UNSIGNED_BYTE, pData);
  end;

  result :=Texture;
end;

procedure SwapRGB(data : PbyteArray; Size,bpp : Integer);
var i : integer;
    C : Byte;
begin
    for I :=0 to Size - 1 do
    begin
      c  := Data[i*bpp+2];
            Data[i*bpp+2] := Data[i*bpp];
            Data[i*bpp  ] := c;
    end;
end;


{------------------------------------------------------------------}
{  Loads 8 , 24 and 32bpp (alpha channel) TGA textures             }
{------------------------------------------------------------------}
function LoadTGATexture(Filename: String; var Texture: Cardinal): Boolean;
var
  TGAHeader : packed record   // Header type for TGA images
    FileType     : Byte;
    ColorMapType : Byte;
    ImageType    : Byte;
    ColorMapSpec : Array [0..4] of Byte;
    OrigX        : Word;
    OrigY        : Word;
    Width        : Word;
    Height       : Word;
    BPP          : Byte;
    ImageInfo    : Byte;
  end;
  TGAFile   : File;
  image     : PbyteArray;    {or PRGBTRIPLE}
  image256  : PbyteArray;
  i,
  ImageSize : Integer;
  Pall      : Array [0..255] of Array [0..2] of Byte;
begin

  if not FileExists(FileName) then
  begin
    {$IfDef Win32}
      MessageBox(0,Pchar('File not found:'+#13+FileName),0,0);
    {$else}
      Writeln('File not found :',FileName);
    {$endif}
    exit;
  end;

  try
    AssignFile(TGAFile, Filename);
    Reset(TGAFile, 1);
    // Read in the bitmap file header
    BlockRead(TGAFile, TGAHeader, SizeOf(TGAHeader));


    // Get the width, height, and color depth
    with TGAHeader do
    IF BPP = 8 then
    begin
    BlockRead(TGAFile, PALL[0][0], 768);

                                     ImageSize  := Width*Height;
                GetMem(image256    , ImageSize);
    BlockRead(TGAFile, image256^[0], ImageSize);

                GetMem(Image       , Width * Height * 3);

       For i := 0 To ImageSize-1 do
       begin
       Image[i*3    ] := Pall[ image256[i] ] [2];
       Image[i*3+1  ] := Pall[ image256[i] ] [1];
       Image[i*3+2  ] := Pall[ image256[i] ] [0];
       end;
       bpp := 24;
     FreeMem(Image256);
    end
    else
    begin
    ImageSize  := Width*Height*(bpp div 8);


    GetMem(Image, ImageSize);
    BlockRead(TGAFile, image^[0], ImageSize);

    SwapRGB(Image, Width * Height, BPP div 8);
    end;
    {$IfDef Win32}
    {$else}
    Writeln(FileName,' ',TGAHeader.Width,' ', TGAHeader.Height, ' ',TGAHeader.BPP);
    {$endif}
    Texture := CreateTexture(TGAHeader.Width, TGAHeader.Height, TGAHeader.BPP, Image);

    FreeMem(Image);
    Result:=True;
   except
    Result:=false;
   end;
end;


end.
