
#include <Extension.h>

#include <string.h>
#include <ctype.h>

#include <SDL/SDL_video.h>

#define GET_PROC_ADDRESS(X) SDL_GL_GetProcAddress(X)

extern "C" {

    static bool _is_ext_supported(const char* ext, const char* extlist)
    {
        const char* start = extlist;
        const char* ptr;
        while ((ptr = strstr(start, ext)) != NULL)
        {
            const char* end = ptr + strlen (ext);
            if (isspace(*end) || *end == '\0')
                return true;
            start = end;
        }
        return false;
    }

    bool isExtSupported(const char* ext)
    {
        const char* extlist = (const char*)glGetString(GL_EXTENSIONS);
        return _is_ext_supported(ext, extlist);
    }


    PFNGLBINDBUFFERARBPROC glBindBufferARB = 0;
    PFNGLDELETEBUFFERSARBPROC glDeleteBuffersARB = 0;
    PFNGLGENBUFFERSARBPROC glGenBuffersARB = 0;
    PFNGLISBUFFERARBPROC glIsBufferARB = 0;
    PFNGLBUFFERDATAARBPROC glBufferDataARB = 0;
    PFNGLBUFFERSUBDATAARBPROC glBufferSubDataARB = 0;
    PFNGLGETBUFFERSUBDATAARBPROC glGetBufferSubDataARB = 0;
    PFNGLMAPBUFFERARBPROC glMapBufferARB = 0;
    PFNGLUNMAPBUFFERARBPROC glUnmapBufferARB = 0;
    PFNGLGETBUFFERPARAMETERIVARBPROC glGetBufferParameterivARB = 0;
    PFNGLGETBUFFERPOINTERVARBPROC glGetBufferPointervARB = 0;

    bool initGL_ARB_vertex_buffer_object()
    {
        glBindBufferARB = (PFNGLBINDBUFFERARBPROC)GET_PROC_ADDRESS("glBindBufferARB");
        glDeleteBuffersARB = (PFNGLDELETEBUFFERSARBPROC)GET_PROC_ADDRESS("glDeleteBuffersARB");
        glGenBuffersARB = (PFNGLGENBUFFERSARBPROC)GET_PROC_ADDRESS("glGenBuffersARB");
        glIsBufferARB = (PFNGLISBUFFERARBPROC)GET_PROC_ADDRESS("glIsBufferARB");
        glBufferDataARB = (PFNGLBUFFERDATAARBPROC)GET_PROC_ADDRESS("glBufferDataARB");
        glBufferSubDataARB = (PFNGLBUFFERSUBDATAARBPROC)GET_PROC_ADDRESS("glBufferSubDataARB");
        glGetBufferSubDataARB = (PFNGLGETBUFFERSUBDATAARBPROC)GET_PROC_ADDRESS("glGetBufferSubDataARB");
        glMapBufferARB = (PFNGLMAPBUFFERARBPROC)GET_PROC_ADDRESS("glMapBufferARB");
        glUnmapBufferARB = (PFNGLUNMAPBUFFERARBPROC)GET_PROC_ADDRESS("glUnmapBufferARB");
        glGetBufferParameterivARB = (PFNGLGETBUFFERPARAMETERIVARBPROC)GET_PROC_ADDRESS("glGetBufferParameterivARB");
        glGetBufferPointervARB = (PFNGLGETBUFFERPOINTERVARBPROC)GET_PROC_ADDRESS("glGetBufferPointervARB");

        return isExtSupported("GL_ARB_vertex_buffer_object");
    }
}
