// g++ -o lesson1 lesson1.cpp -lGL -lGLU -lSDL -lSDL_image
#include <GL/gl.h>
#include <GL/glu.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include <iostream>
#include <fstream>
#include <string>
#include <list>
using namespace std;

#include <Extension.h>

/// NEW
GLfloat filter;

// Render one frame
void prepareFrame();
void renderFrame();

// Timer
Uint32 myTimerCallback(Uint32 interval, void*);
void* myTimerCallbackParam;

// Projection managing
void setPerspective();
void setOrtho();

int fps = 0;
float lastTime = 0.0f;
float currentTime = 0.0f;
float cubePos = 0.0f;
float cubeRot = 0.0f;

bool keys[512];

// Camera position
float camX = 0.0f, camY = 10.0f, camZ = 0.0f;

// Object moving speed
float speed = 0.03f;

// Cube vertices
float cubeVertices[] =
{
    1.0f, 1.0f,-1.0f,
    -1.0f, 1.0f,-1.0f,
    -1.0f, 1.0f, 1.0f,
    1.0f, 1.0f, 1.0f,

    1.0f,-1.0f, 1.0f,
    -1.0f,-1.0f, 1.0f,
    -1.0f,-1.0f,-1.0f,
    1.0f,-1.0f,-1.0f,

    1.0f, 1.0f, 1.0f,
    -1.0f, 1.0f, 1.0f,
    -1.0f,-1.0f, 1.0f,
    1.0f,-1.0f, 1.0f,

    1.0f,-1.0f,-1.0f,
    -1.0f,-1.0f,-1.0f,
    -1.0f, 1.0f,-1.0f,
    1.0f, 1.0f,-1.0f,

    -1.0f, 1.0f, 1.0f,
    -1.0f, 1.0f,-1.0f,
    -1.0f,-1.0f,-1.0f,
    -1.0f,-1.0f, 1.0f,

    1.0f, 1.0f,-1.0f,
    1.0f, 1.0f, 1.0f,
    1.0f,-1.0f, 1.0f,
    1.0f,-1.0f,-1.0f
};


// Cube texcoords
float cubeTexcoords [] =
{
    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, -1.0f,
    0.0f, -1.0f,

    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, -1.0f,
    0.0f, -1.0f,

    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, -1.0f,
    0.0f, -1.0f,

    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, -1.0f,
    0.0f, -1.0f,

    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, -1.0f,
    0.0f, -1.0f,

    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, -1.0f,
    0.0f, -1.0f,
};

// Plane vertices
float planeVertices[] =
{
    100.0f, -1.0f, 100.0f,
    100.0f, -1.0f, -100.0f,
    -100.0f, -1.0f, -100.0f,
    -100.0f, -1.0f, 100.0f
};

// Plane texcoords
float planeTexcoords[] =
{
    0.0f, 0.0f,
    8.0f, 0.0f,
    8.0f, -8.0f,
    0.0f, -8.0f
};

GLuint loadTexture(const string& filename, bool aniso);
GLuint cubeTexture, planeTexture;
GLuint cubeDetailmap, planeDetailmap;

#define NUM_SPHERE_VERTICES 90000
bool useVBO = true;
GLfloat sphereVertexArray[NUM_SPHERE_VERTICES * 3];
GLuint sphereVertexBuffer;
void genSphere();
void drawSphere();

#define IEEE_1_0 0x3f800000

inline float rsqrt(const float x)
{
    unsigned tmp = (unsigned(IEEE_1_0 << 1) + IEEE_1_0 - *(unsigned*)&x) >> 1;
    float y = *(float*)&tmp;
    return y * (1.47f - 0.47f * x * y * y);
}

struct Sphere
{
    float x, y, z,
        r, g, b;
};

list<Sphere*> spheres;

void cleanUp()
{
    if (cubeTexture)
    {
        glDeleteTextures(1, &cubeTexture);
        cout << "Cube texture deleted"
            << endl;
    }
    if (cubeDetailmap)
    {
        glDeleteTextures(1, &cubeDetailmap);
    }
    if (planeTexture)
    {
        glDeleteTextures(1, &planeTexture);
        cout << "Plane texture deleted"
            << endl;
    }
    if (planeDetailmap)
    {
        glDeleteTextures(1, &planeDetailmap);
    }
    SDL_Quit();
}
int main()
{
    if (SDL_Init(SDL_INIT_VIDEO|
                 SDL_INIT_TIMER) < 0)
    {
        cout << "Cannot init SDL: "
            << SDL_GetError() << endl;
        exit(1);
    }

    atexit(cleanUp);

    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, true);
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 6);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);

	if (SDL_SetVideoMode(800, 600, 16, SDL_OPENGL) == NULL)
	{
	    cout << "Cannot set video mode: "
            << SDL_GetError() << endl;
        exit(1);
    }

    SDL_WM_SetCaption("My Cool Linux Game!", "game");
    SDL_WM_SetIcon(IMG_Load("icon.jpg"), NULL);

    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    glShadeModel(GL_SMOOTH);

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    if (!isExtSupported("GL_ARB_vertex_buffer_object"))
    {
        cout << "Error: No VBO support!"
            << endl;
        exit(1);
    }

    if (!isExtSupported("GL_EXT_texture_filter_anisotropic"))
    {
        cout << "Error: No Aniso support!"
            << endl;
        exit(1);
    }

    if (!isExtSupported("GL_ARB_multitexture"))
    {
        cout << "Error: Mo Multitexture support!"
            << endl;
        exit(1);
    }

    initGL_ARB_vertex_buffer_object();

    glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &filter);
    cout << "Anisotropy: " << filter << "x" << endl;

    genSphere();

    setPerspective();

    cubeTexture = loadTexture("textures/cube-texture.jpg", true);
    cubeDetailmap = loadTexture("textures/detailmap2.jpg", false);
    planeTexture = loadTexture("textures/plane-texture.jpg", true);
    planeDetailmap = loadTexture("textures/detailmap3.jpg", false);

    currentTime = SDL_GetTicks();

    // Setup timer
    SDL_AddTimer(1000, myTimerCallback, myTimerCallbackParam);

    // Enter main loop
    bool done = false;
    SDL_Event event;
    while (!done)
    {
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
                case SDL_QUIT:
                    done = true;
                    break;
                case SDL_KEYDOWN:
                    keys[event.key.keysym.sym] = true;
                    break;
                case SDL_KEYUP:
                    if (event.key.keysym.sym == SDLK_ESCAPE)
                        done = true;
                    if (event.key.keysym.sym == SDLK_TAB)
                        useVBO = !useVBO;
                    keys[event.key.keysym.sym] = false;
                    break;
                default:
                    break;
            }
        }

        prepareFrame();
        renderFrame();
        SDL_GL_SwapBuffers();
        fps++;
    }
    return 0;
}

void prepareFrame()
{
    float elapsedTime = (currentTime - lastTime) / 1000;

    if (cubePos >= 100.0f || cubePos <= -100.0f)
        speed = -speed;

    cubePos -= elapsedTime * speed;

    cubeRot = (cubeRot >= 360.0f)
    ? 0.0f : cubeRot + elapsedTime * 0.1f;

    if (keys['w'])
        camZ -= elapsedTime * 0.1f;
    if (keys['s'])
        camZ += elapsedTime * 0.1f;
    if (keys['a'])
        camX -= elapsedTime * 0.1f;
    if (keys['d'])
        camX += elapsedTime * 0.1f;

    // Zoom camera in
    if (keys['e'])
    {
        if (camY <= 30.0f)
        {
            camY += elapsedTime * 0.1f;
            camZ += elapsedTime * 0.1f;
        }
    }
    // Zoom camera out
    if (keys['r'])
    {
        if (camY >= 1.0f)
        {
            camY -= elapsedTime * 0.1f;
            camZ -= elapsedTime * 0.1f;
        }
    }
}

void renderFrame()
{
//    prepareFrame();
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    setPerspective();
    // Draw plane
    glPushMatrix();

        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, planeTexture);

        glEnableClientState(GL_VERTEX_ARRAY);
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
        
        
        glVertexPointer(3, GL_FLOAT, 0, planeVertices);

        glTexCoordPointer(2, GL_FLOAT, 0, planeTexcoords);


        glDrawArrays(GL_QUADS, 0, 4);
        glDisableClientState(GL_VERTEX_ARRAY);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
        
        glBindTexture(GL_TEXTURE_2D);
        glDisable(GL_TEXTURE_2D);
        
    glPopMatrix();
    // Draw moving cube
    glPushMatrix();
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, cubeTexture);
        glTranslatef(0.0f, 0.0f, cubePos);
        glRotatef(cubePos, 0.0f, 1.0f, 0.0f);
        glEnableClientState(GL_VERTEX_ARRAY);
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
        glVertexPointer(3, GL_FLOAT, 0, cubeVertices);
        glTexCoordPointer(2, GL_FLOAT, 0, cubeTexcoords);
        glDrawArrays(GL_QUADS, 0, 24);
        glDisableClientState(GL_VERTEX_ARRAY);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glPopMatrix();
    // Draw rotating cube
    glPushMatrix();
    glTranslatef(-50.0f, 0.0f, 0.0f);
    glPushMatrix();
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, cubeTexture);

        glRotatef(cubeRot, 0.0f, 1.0f, 0.0f);
        glTranslatef(0.0f, 0.0f, 20.0f);


        glEnableClientState(GL_VERTEX_ARRAY);
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
        glVertexPointer(3, GL_FLOAT, 0, cubeVertices);
        glTexCoordPointer(2, GL_FLOAT, 0, cubeTexcoords);
        glDrawArrays(GL_QUADS, 0, 24);
        glDisableClientState(GL_VERTEX_ARRAY);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glPopMatrix();
    glPopMatrix();

    // Draw spheres
    glPushMatrix();

    for (list<Sphere*>::iterator it = spheres.begin();
        it != spheres.end(); it++)
    {
        glPushMatrix();

        glTranslatef((*it)->x, (*it)->y, (*it)->z);

        glColor3f((*it)->r, (*it)->g, (*it)->b);
        drawSphere();
        glColor3f(1, 1, 1);

        glPopMatrix();
    }

    glPopMatrix();

    setOrtho();
    // Here we shall draw some text...
}

Uint32 myTimerCallback(Uint32 interval, void*)
{
    cout << "FPS = "
        << fps << endl;
    fps = 0;
    lastTime = currentTime;
    currentTime = SDL_GetTicks();
    return interval;
}

void setPerspective()
{
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, 800, 600);
	gluPerspective(60.0f, GLfloat(800/600), 0.1f, 3000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glEnable(GL_DEPTH_TEST);
	// Prepare camera
	glRotatef(30.0f, 1.0f, 0.0f, 0.0f);
	glTranslatef(-camX, -camY, -camZ);
}

void setOrtho()
{
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, 800, 600);
	glOrtho(0, 800, 600, 0, 0, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glDisable(GL_DEPTH_TEST);
}


GLuint loadTexture(const string& name, bool aniso)
{
    cout << "Loading texture: "
		<< name << endl;
	string e = name.substr(name.size() - 3, 3);
	GLenum type = (e == "tga") ? GL_BGR : GL_RGB;
	GLuint texture;
	glPixelStorei(GL_UNPACK_ALIGNMENT, 8);
	SDL_Surface* surface = IMG_Load(name.c_str());
	if (surface)
	{
		glEnable(GL_TEXTURE_2D);
		glGenTextures(1, &texture);
		glBindTexture(GL_TEXTURE_2D, texture);

        if (aniso)
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, filter);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                        GL_LINEAR_MIPMAP_LINEAR);

        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, type,
                          GL_UNSIGNED_BYTE, surface->pixels);
        SDL_FreeSurface(surface);
        return texture;
	}
	cout << "Failed to load texture, using blank instead."
		<< endl;
	return 0;
}

void genSphere()
{
    for (size_t i = 0; i < NUM_SPHERE_VERTICES; i++)
    {
        GLfloat r1, r2, r3, scaleFactor;
        r1 = (GLfloat)(rand() - (RAND_MAX / 2));
        r2 = (GLfloat)(rand() - (RAND_MAX / 2));
        r3 = (GLfloat)(rand() - (RAND_MAX / 2));

        scaleFactor = rsqrt(r1*r1 + r2*r2 + r3*r3);
        sphereVertexArray[(i * 3)] = r1 * scaleFactor;
        sphereVertexArray[(i * 3) + 1] = r2 * scaleFactor;
        sphereVertexArray[(i * 3) + 2] = r3 * scaleFactor;
    }

    glGenBuffersARB(1, &sphereVertexBuffer);
    glBindBufferARB(GL_ARRAY_BUFFER_ARB, sphereVertexBuffer);
    glBufferDataARB(GL_ARRAY_BUFFER_ARB, sizeof(GLfloat) * NUM_SPHERE_VERTICES * 3,
        sphereVertexArray, GL_STATIC_DRAW_ARB);

    glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);

    ifstream inf("spheres");
    if (!inf.is_open())
    {
        cout << "Data file NOT found!"
            << endl;
        exit(1);
    }
    while (!inf.eof())
    {
        Sphere* sphere = new Sphere;
        inf >> sphere->x >> sphere->y >> sphere->z
            >> sphere->r >> sphere->g >> sphere->b;
        //
        spheres.push_back(sphere);
    }
    inf.close();
}

void drawSphere()
{
    if (useVBO)
    {
        glBindBufferARB(GL_ARRAY_BUFFER_ARB, sphereVertexBuffer);
        glNormalPointer(GL_FLOAT, 0, 0);
        glVertexPointer(3, GL_FLOAT, 0, 0);
    }
    else
    {
        glNormalPointer(GL_FLOAT, 0, sphereVertexArray);
        glVertexPointer(3, GL_FLOAT, 0, sphereVertexArray);
    }

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);

    glDrawArrays(GL_POINTS, 0, NUM_SPHERE_VERTICES);

    glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);

    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
}
